Package: Polychrome
Title: Qualitative Palettes with Many Colors
Version: 1.5.4
Date: 2025-04-06
Authors@R: c(person(given = "Guy", family = "Brock", role = "aut"),
	    person(given = c("Kevin", "R."), family = "Coombes",
                   role = c("aut", "cre"), email = "krc@silicovore.com"))
Description: Tools for creating, viewing, and assessing qualitative
 palettes with many (20-30 or more) colors. See Coombes and colleagues
 (2019) <doi:10.18637/jss.v090.c01>.
Depends: R (>= 4.4)
Imports: colorspace, scatterplot3d, methods, graphics, grDevices,
        stats, utils
Suggests: RColorBrewer, knitr, rmarkdown, ggplot2
License: Apache License (== 2.0)
LazyLoad: yes
LazyData: no
URL: http://oompa.r-forge.r-project.org/
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-04-06 21:14:05 UTC; kevin
Author: Guy Brock [aut],
  Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: CRAN
Date/Publication: 2025-04-06 21:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 11:36:34 UTC; unix

Package: Polychrome
Title: Qualitative Palettes with Many Colors
Version: 1.6.1
Date: 2026-07-06
Authors@R: c(person(given = "Guy", family = "Brock", role = "aut"),
	    person(given = c("Kevin", "R."), family = "Coombes",
                   role = c("aut", "cre"), email = "krc@silicovore.com"))
Description: Tools for creating, viewing, and assessing qualitative
 palettes with many (20-30 or more) colors. See Coombes and colleagues
 (2019) <doi:10.18637/jss.v090.c01>.
Depends: R (>= 4.4)
Imports: colorspace, scatterplot3d, methods, graphics, grDevices,
        stats, utils
Suggests: RColorBrewer, knitr, rmarkdown, ggplot2
License: Apache License (== 2.0)
LazyLoad: yes
LazyData: no
URL: http://oompa.r-forge.r-project.org/
VignetteBuilder: knitr
Author: Guy Brock [aut],
  Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: R-Forge
Repository/R-Forge/Project: oompa
Repository/R-Forge/Revision: 708
Repository/R-Forge/DateTimeStamp: 2026-07-06 14:32:48
Date/Publication: 2026-07-06 14:32:48
NeedsCompilation: no
Packaged: 2026-07-06 14:44:13 UTC; rforge

###
### NAMESPACE
###

## Imported methods
import(colorspace)
import(scatterplot3d)

importFrom("graphics", "barplot", "lines", "mtext", "text", "par", "abline", "image")
importFrom("grDevices", "col2rgb", "colors", "rgb")
importFrom("methods", "as")
importFrom("stats", "dist", "hclust", "rnorm", "runif", "princomp")
importFrom("utils", "data")

## Public functions
export(rancurves, ranpoints, uvscatter, luminance,
       plothc, plotpc, p3d, swatch, swatchHue, swatchLuminance, ranswatch,
       computeDistances, plotDistances, invertColors,
       createPalette,
       colorNames, isccNames, xkcdNames)
export(getLUV)

export(sortByHue, sortByLuminance)

export(colorDeficit)

export(memberPlot)

# public data sets
export(kelly.colors, green.armytage.colors,
       alphabet.colors, glasbey.colors,
       palette36.colors, light.colors, dark.colors,
       sky.colors)

temp <- read.table("temp.txt", sep="\t", comment.char='', as.is=TRUE)

foo <- strsplit(temp$V1, "\\s")
ping <- unlist(lapply(foo, function(f) which(!is.na(as.integer(f)))))
counter <- as.integer(sapply(1:length(foo), function(i) {
  foo[[i]][ping[i]]
}))
summary(diff(counter)) # all 1's

shortName <- unlist(sapply(1:length(foo), function(i) {
  paste(foo[[i]][1:(ping[i]-1)], collapse="_")
}))

pong <- unlist(lapply(foo, function(f) {
  w <- which(f %in% c('see', 'not'))
  ifelse(length(w) > 0, w-1, length(f))
}))

longName <- unlist(sapply(1:length(foo), function(i) {
  f <- foo[[i]]
  paste(f[(ping[i]+1):pong[i]], collapse="_")
}))
ln <- strsplit(longName, ',')
longName <- unlist(lapply(ln, function(x) x[1]))

iscc <- data.frame(shortName, longName, Hex=temp$V2)
write.table(iscc, file="iscc.txt", sep="\t", row.names=FALSE, quote=FALSE)
save(iscc, file="iscc.rda")

library(Polychrome)
library(colorspace)

####################################
# palette36

## should not be called by user code
createnew36 <- function() {
  # three seed colors
  ebony <- hex(LUV(30,0,0))
  iron <- hex(LUV(90, 0, 0))
  red <- hex(sRGB(0.8, 0, 0))
  # set random number seed for reproducibility. 
  set.seed(567629)
  createPalette(36, c(ebony, iron, red))
}
palette36 <- createnew36()
names(palette36) <- isccNames(palette36)
save(palette36, file="palette36.rda")

####################################
# alphacolors

createAlphabet <- function() {
  alphabet <- palette36[1:26]
  # want to assign not-completely-stupid names
  # start with names from Green-Armytage
  colsch <- green.armytage.colors()
  ga <- as(hex2RGB(colsch), "LUV")
  alpha <- as(hex2RGB(alphabet), "LUV")
  d3 <- function(y0) {
    temp <- sweep(ga@coords, 2, y0, "-")
    dist <- apply(temp^2, 1, sum)
    which(dist==min(dist))
  }
  m <- apply(alpha@coords, 1, d3)
  odd <- which(!(1:26 %in% m))
  m[duplicated(m)] <- odd
  names(alphabet) <- names(colsch)[m]
  # now manually swap things around
  # I know; this is really dumb.
  b <- which(names(alphabet)=='blue')
  v <- which(names(alphabet)=='violet')
  tu <- which(names(alphabet)=='turquoise')
  names(alphabet)[b] <- "turquoise"
  names(alphabet)[tu] <- "violet"
  names(alphabet)[v] <- "blue"
  rm(b, v, tu)
  s <- which(names(alphabet) == "sky")
  g <- which(names(alphabet) == "green")
  j <- which(names(alphabet) == "jade")
  k <- which(names(alphabet) == "khaki")
  o <- which(names(alphabet) == "orpiment")
  w <- which(names(alphabet) == "wine")
  names(alphabet)[g] <- "sea"
  names(alphabet)[k] <- "jade"
  names(alphabet)[o] <- "green"
  names(alphabet)[s] <- "wine"
  names(alphabet)[j] <- "O"
  names(alphabet)[w] <- "kingcrab"
  rm(s,g,o,k, j)
  names(alphabet)[names(alphabet)=="zinnia"] <- "orange"
  names(alphabet)[names(alphabet)=="lime"] <- "zinnia"
  names(alphabet)[names(alphabet)=="amethyst"] <- "lavender"
  names(alphabet)[names(alphabet)=="O"] <- "amethyst"
  names(alphabet)[names(alphabet)=="mallow"] <- "magenta"
  names(alphabet)[names(alphabet)=="damson"] <- "ultraviolet"
  names(alphabet)[names(alphabet)=="uranium"] <- "damson"
  x <- which(names(alphabet) == "xanthin")
  h <- which(names(alphabet) == "honeydew")
  names(alphabet)[x] <- "honey"
  names(alphabet)[h] <- "xanthin"
  rm(x,h)
  alphabet["ebony"] <- "#565656"
  alphabet["iron"] <- "#E2E2E2"
  alphabet[order(names(alphabet))]
}
alphabet <- createAlphabet()

save(alphabet, file="alphabet.rda")

## ----opts, echo=FALSE---------------------------------------------------------
knitr::opts_chunk$set(fig.width=8, fig.height=5)

## ----pkgs---------------------------------------------------------------------
library(Polychrome)

## ----rg, fig.cap="Common red-green palette."----------------------------------
rg <- colorRampPalette(c("red", "black", "green"))(64)
image(matrix(1:64), col=rg)

## ----deut, fig.cap="Deuteranopes' view of the common red-green palette."------
rg.deut <- colorDeficit(rg, "deut")
image(matrix(1:64), col=rg.deut)

## ----prot, fig.cap="Protanopes' view of the common red-green palette."--------
rg.prot <- colorDeficit(rg, "prot")
image(matrix(1:64), col=rg.prot)

## ----trit, fig.cap="Tritanopes' view of the common red-green palette."--------
rg.trit <- colorDeficit(rg, "trit")
image(matrix(1:64), col=rg.trit)

## ----alfa, fig.cap = "The 26-color alphabet palette."-------------------------
alfa <- alphabet.colors(26)
swatch(alfa)

## ----adeut, fig.cap = "The 26-color alphabet palette, as seen by a deuteranope."----
alfa.deut <- colorDeficit(alfa, "deut")
swatch(alfa.deut)

## ----aprot, fig.cap = "The 26-color alphabet palette, as seen by a protanope."----
alfa.prot <- colorDeficit(alfa, "prot")
swatch(alfa.prot)

## ----atrit, fig.cap = "The 26-color alphabet palette, as seen by a tritanope."----
alfa.trit <- colorDeficit(alfa, "trit")
swatch(alfa.trit)

## ----cp1, fig.cap="A palette designed for deuteranopes."----------------------
cp <- createPalette(20, "#111111", target="deuteranope")
names(cp) <- colorNames(cp)
swatch(cp)

## ----cp2, fig.cap = "A palette designed for protanopes."----------------------
cp <- createPalette(20, "#111111", target="protanope")
names(cp) <- colorNames(cp)
swatch(cp)

## ----cp3, fig.cap = "A palette designed for tritanopes."----------------------
cp <- createPalette(20, "#111111", target="tritanope")
names(cp) <- colorNames(cp)
swatch(cp)

## ----dists--------------------------------------------------------------------
p34 <- palette36.colors(36)[3:36]
names(p34) <- colorNames(p34)
p34.deut <- colorDeficit(p34, "deut")
p34.prot <- colorDeficit(p34, "prot")
p34.trit <- colorDeficit(p34, "trit")

## ----shift--------------------------------------------------------------------
shift <- function(i, k=34) c(i, 1:(i-1), (1+i):k)
co <- shift(13)
pd <- computeDistances(p34.deut[co])
pp <- computeDistances(p34.prot[co])
pt <- computeDistances(p34.trit[co])

## ----score--------------------------------------------------------------------
rd <- rank(pd)[order(names(pd))]
rp <- rank(pd)[order(names(pp))]
rt <- rank(pd)[order(names(pt))]
score <- 2*rd + 1.5*rp + rt

## ----fig.width=8, fig.height=7, fig.cap="Color safe palette."-----------------
x <- p34[names(rev(sort(score)))][1:10]
y <- colorDeficit(x, "deut")
z <- colorDeficit(x, "prot")
w <- colorDeficit(x, "trit")

opar <- par(mfrow=c(2,2))
swatch(x, main="Normal")
swatch(y, main="Deuteranope")
swatch(z, main="Protanope")
swatch(w, main="Tritanope")

## ----echo=FALSE---------------------------------------------------------------
par(opar)

## -----------------------------------------------------------------------------
evalVignette <- requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(eval = evalVignette)

## ----newpal, fig.cap="A new palette of 40 colors.", fig.width = 8, fig.height=4----
library(Polychrome)
set.seed(935234)
P40 <- createPalette(40, c("#FF0000", "#00FF00", "#0000FF"), range = c(30, 80))
swatch(P40)

## ----reorder, fig.cap="A sorted palette of 40 colors.", fig.width = 8, fig.height=4----
P40 <- sortByHue(P40)
P40 <- as.vector(t(matrix(P40, ncol=4)))
swatch(P40)

## ----dename-------------------------------------------------------------------
names(P40) <- NULL

## ----simdata------------------------------------------------------------------
## Nine groups
NG <- 9
gp <- paste("G", 1:NG, sep = "")
length(gp)

## Four Subjects per group
## 36 Subjects = 9 groups * 4 subjects/group 
sid <- paste(rep(LETTERS[1:2], each=26), c(LETTERS, LETTERS), sep="")[1:(4*NG)]
length(sid)

## Three Reps per subject
## 108 Experiments
reps = factor(rep(c("R1", "R2", "R3"), times = length(sid)))
length(reps)

## Each experiment with measurements on four Days, so 432 data rows
daft <- data.frame(Day = rep(1:4, each=length(reps)),
                   Group = factor(rep(rep(gp, each=12), times = 4)),
                   Subject = factor(rep(rep(sid, each = 3), times=4)),
                   Rep = factor(rep(reps, times = 4)))
dim(daft)
summary(daft)

## ----variable-----------------------------------------------------------------
## Linear model with noise, ignoring group
beta <- runif(length(sid), 0.5, 2)
## "Measured" variable
attach(daft)
daft$variable <- rnorm(nrow(daft), 0, 0.2) + 1  + beta[as.numeric(Subject)]*Day
detach()

## ----fig.cap="A faceted plot, colored by subject.", fig.width=8, fig.height=8----
library(ggplot2)
ggplot(daft, aes(x = Day, y = variable, colour = as.factor(Subject))) +
  geom_point(aes(shape = as.factor(Rep)), size = 3) +
  geom_line(aes(linetype = as.factor(Rep)), size = 0.8) +
  facet_wrap(. ~ Group, ncol = 3)+
  theme_bw() + theme(legend.position="none")+
  scale_color_manual(values = P40)

# An alternative set of a dozen safe colors.
library(Polychrome)
set.seed(4528)
tempPal <- createPalette(40, c("#010101", "#FFFFFF"), range=c(25, 80))
tempPal <- tempPal[-(1:2)]
swatch(tempPal)

tempPal.deut <- colorDeficit(tempPal, "deut")
tempPal.prot <- colorDeficit(tempPal, "prot")
tempPal.trit <- colorDeficit(tempPal, "trit")

shift <- function(i, k=length(tempPal)) c(i, 1:(i-1), (1+i):k)
co <- shift(13)
pd <- computeDistances(tempPal.deut[co])
pp <- computeDistances(tempPal.prot[co])
pt <- computeDistances(tempPal.trit[co])

rd <- rank(pd)[order(names(pd))]
rp <- rank(pd)[order(names(pp))]
rt <- rank(pd)[order(names(pt))]
score <- 2*rd + 1.5*rp + rt

x <- tempPal[names(rev(sort(score)))][1:12]
plotDistances(x,cex=2)
y <- colorDeficit(x, "deut")
z <- colorDeficit(x, "prot")
w <- colorDeficit(x, "trit")

opar <- par(mfrow=c(2,2))
swatch(x, main="Normal")
swatch(y, main="Deuteranope")
swatch(z, main="Protanope")
swatch(w, main="Tritanope")
par(opar)

names(x) <- colorNames(x)
safeColors <- x

plotDistances(safeColors,cesafeColors=2)

opar <- par(mfrow=c(2,2))
swatch(safeColors)
swatchHue(safeColors)
swatchLuminance(safeColors)
ranswatch(safeColors)
par(opar)

rancurves(safeColors, lwd=3)
ranpoints(safeColors, cesafeColors=1.5)
uvscatter(safeColors)
luminance(safeColors)
plothc(safeColors)
plotpc(safeColors)

library(Polychrome)
set.seed(9641)
Dark24 <- createPalette(24, c("#2A95E8", "#E5629C"), range = c(10, 60),
   M = 100000)

set.seed(701138)
Light24 <- createPalette(24, "#ff0000", range = c(55, 95), M = 100000)

ddir <- file.path("..", "..", "data")
save(Dark24, file = file.path(ddir, "Dark24.rda"))
save(Light24, file = file.path(ddir, "Light24.rda"))

set.seed(9641)
dark24 <- createPalette(24, c("#2A95E8", "#E5629C" ), range = c(10, 60), M = 100000)
chrColors <- rev(dark24)
#set.seed(9641)
#chrColors <- dark24[c(sample(c(1:16, 18:23)), 24, 17)]
names(chrColors) <- paste("chr", c(1:22, "X", "Y"))

amlPC <- read.csv(system.file("extData/amlPC.csv", package = "Polychrome"), row.names = 1)

makeleg <- function(x, y, dx, dy = 0, chrColors, pch) {
  lab <- c(1:22, "X", "Y")
  legend(x, y, lab[1:8], col = chrColors[1:8], pch=15)
  legend(x + dx, y + dy, lab[9:16], col = chrColors[9:16], pch=pch)
  legend(x + 2*dx, y + 2*dy, lab[17:24], col = chrColors[17:24], pch=pch)
}

plot(amlPC[,2], amlPC[,3],
     pch=15, col=chrColors[amlPC[,1]],
     xlab="Loadings, PC1", ylab="Loadings, PC2")
makeleg(0.05, -0.07, 0.01, 0, chrColors, 15)

f <- function(i) {
  pick <- amlPC[,1] == i
  points(amlPC[pick,2], amlPC[pick,3], col=chrColors[i], cex=2)
}

f(1)
f(2)
f(3)

f(24)

##########################################
## manual conversion between sRGB and RGB
srgb2rgb <- function(x) {
  a <- 0.055
  ifelse(x <= 0.04045,
         x/12.92,
         ((x + a)/(1+a))^2.4)
}

rgb2srgb <- function(x) {
  a <- 0.055
  ifelse(x <= 0.0031308,
         12.92*x,
         (1+a)*x^(1/2.4) - a)
}

##########################################
## Transformation matrices

## convert RGB to XYZ space (CIE uses 4-digit approxiomations)
RGB2XYZ <- matrix(c(0.4124, 0.3576, 0.1805,
              0.2126, 0.7152, 0.0722,
              0.0193, 0.1192, 0.9505),
            3, 3, byrow=TRUE)
XYZ2RGB <- round(solve(RGB2XYZ), 4)
## they also write (X, Y, Z)^t = M * (R, G, B)^t

## equal energy; rows sum to 1
eeXYZ2LMS <- matrix(c( 0.38971, 0.68898, -0.07868,
                      -0.22981, 1.18340,  0.04641,
                      0, 0, 1), 3, 3, byrow=TRUE)
eeLMS2XYZ <- round(solve(eeXYZ2LMS), 5)
## normalized to D65 daylight illumination
d65XYZ2LMS <- matrix(c( 0.4002, 0.7076, -0.0808,
                       -0.2263, 1.1653, 0.0457,
                      0, 0, 1), 3, 3, byrow=TRUE)
d65LMS2XYZ <- round(solve(d65XYZ2LMS), 5)


##########################################
## We assume that teh default behavior is to convert
## sRGB coordinates to (erqual energy) LMS coordinates
forward <- function(x) {
  srgb2rgb(x) %*% t(RGB2XYZ) %*% t(eeXYZ2LMS)
}

## the round trip isn't completely symmetric, since we
## have to truncate to get back into the 0-1 limitations
## of visible space
backward <- function(x) {
  y <- rgb2srgb(x %*% t(eeLMS2XYZ) %*% t(XYZ2RGB) )
  y[y < 0] <- 0
  y[y > 1] <- 1
  y
}

##########################################
## Color deficit transformation matrices

## Deuteranope (most common form of red/green color deficit)
lms2lmsd <- matrix(c(1,        0, 0,
                     0.494207, 0, 1.24827,
                     0,        0, 1), 3, 3, byrow=TRUE)

## Protanope (another form of red/green color deficit)
lms2lmsp <- matrix(c(0, 2.02344, -2.52581,
                     0, 1,        0,
                     0, 0,        1), 3, 3, byrow=TRUE)

## Transformation matrix for Tritanope (a blue/yellow deficit - very rare)
lms2lmst <- matrix(c(1,         0,        0,
                     0,         1,        0,
                     -0.395913, 0.801109, 0), 3, 3, byrow=TRUE)

##########################################


colorDeficit <- function(rgb,
                      target = c("deuteranope", "protanope", "tritanope")) {
### handle different kinds of inputs
  ## start with a class from 'colorspace'
  if (returnObject <- inherits(rgb, "sRGB")) {
    obj <- rgb
    rgb <- rgb@coords
  }
  ## character strings are assumed to be hexadecimnal colors
  if (returnHex <- is.character(rgb)) {
    obj <- hex2RGB(rgb)
    rgb <- obj@coords
  }

### What kind of transformation do we want?
  target <- match.arg(target)
  xform <- switch(target,
                  deuteranope = lms2lmsd,
                  protanope = lms2lmsp,
                  tritanope = lms2lmst
                  )
### MAIN ALGORITHM
  deficit <- backward(forward(rgb) %*% t(xform))

### Give the user what (s)he wants.
  if (returnHex | returnObject) {
    obj@coords <- deficit
    deficit <- obj
  }
  if (returnHex) deficit <- hex(deficit)
  deficit
}

library(Polychrome)

tem <- tema <- 1:12
names(tem) <- paste("C", 1:12, sep='')
all(Polychrome:::xform(tem) == Polychrome:::xform(tem))

tem <- topo.colors(12, alpha=0.6)
all(tem == Polychrome:::xform(tem, alpha=TRUE))

suppressWarnings( RNGversion("3.5.0") )
set.seed(236746)
tem <- sample(colors(), 12)
Polychrome:::xform(tem)
Polychrome:::xform(tem, alpha=TRUE)

### make sure we just have color names here to test the auto conversion
tem

swatch(tem)
ranswatch(tem)
rancurves(tem)
ranpoints(tem)
uvscatter(tem)
luminance(tem)
plothc(tem)
plotpc(tem)
swatchHue(tem)
swatchLuminance(tem)

round(computeDistances(tem), 1)
plotDistances(tem, pch=16)

colorNames(tem)
isccNames(tem)

library(Polychrome)
library(colorspace)

#########################################
## Test the background algorithm

## checking the conversion between RGB and sRGB
y <- seq(0, 1, length=500)
liny <- Polychrome:::srgb2rgb(y)
plot(y, liny)

yback <- Polychrome:::rgb2srgb(liny)
plot(y, yback)
abline(0,1)
summary(y - yback)

## comparing with colorspace

p36 <- palette36.colors(36)
srgb <- hex2RGB(p36)
mat <- srgb@coords
linrgb <- as(srgb, "RGB")
linmat <- Polychrome:::srgb2rgb(mat)
summary(linmat - linrgb@coords) # all 0, so we are doing the same thing

## since our structures have row vectors instead of columns,
## we need to transpose stuff
xyz <- linmat %*% t(Polychrome:::RGB2XYZ)
rgb.back <- xyz %*% t(Polychrome:::XYZ2RGB)
summary(round(linmat - rgb.back, 3)) # accurate to 4 decimal places

## multiple options for convertiong XYZ spoace to LMS perceptual space

## check that these are projections
all(Polychrome:::lms2lmsd == Polychrome:::lms2lmsd %*% Polychrome:::lms2lmsd)
all(Polychrome:::lms2lmsp == Polychrome:::lms2lmsp %*% Polychrome:::lms2lmsp)
all(Polychrome:::lms2lmst == Polychrome:::lms2lmst %*% Polychrome:::lms2lmst)

## check the complete round trip
M <- Polychrome:::backward(Polychrome:::forward(mat))
summary(round(M - mat, 3))


#########################################
## Test user-visible commands

## three input types
p36 <- palette36.colors(36)
srgb <- hex2RGB(p36)
class(srgb)
mat <- srgb@coords

## check that each one works
daltMat <- colorDeficit(mat, target="deut") # matrix conversion
daltRGB <- colorDeficit(srgb, target="deut") # object conversion
all(daltMat == daltRGB@coords) # agreement

daltP36 <- colorDeficit(p36, "deut") # hex string conversion
all(daltP36 == hex(daltRGB)) # agreement

# invariance
reup <- colorDeficit(daltMat, "deut")
all(daltMat == reup) # why false?
all(abs(daltMat - reup) < 0.4)

sndP36 <- hex(new("sRGB", coords=reup))

opar <- par(mfrow=c(3,1))
swatch(p36)
swatch(daltP36)
swatch(sndP36)
par(opar)

swatchHue(daltP36)
uvscatter(daltP36)
luminance(daltP36)
plotDistances(daltP36, cex=2)


cp <- createPalette(25, c("#111111", "#eeeeee"),
                    target="deuteranope", M=10000)
swatch(cp)
plotDistances(cp)

cp <- createPalette(25, c("#111111", "#eeeeee"),
                    target="prot", M=10000)
swatch(cp)
plotDistances(cp)

cp <- createPalette(25, c("#111111", "#eeeeee"),
                    target="trit", M=10000)
swatch(cp)
plotDistances(cp)

library(Polychrome)

try( memberPlot( rep(0:1, 20)))                       # must be a matrix
try( memberPlot(matrix(LETTERS, ncol = 2 )) )         # must be numeric
try( memberPlot(matrix(1:30, ncol = 5), pal = palette()) ) # and binary

temp <- t(data.frame(
  A = c(rep(1, 500), rep(0, 100)),
  B = c(rep(1, 500), rep(0, 100)),
  C = c(rep(1, 300), rep(0, 300)),
  D = c(rep(1, 300), rep(0, 300)),
  E = c(rep(1, 300), rep(0, 300)),
  F = c(rep(1, 180), rep(0, 320), rep(1, 40), rep(0, 60)),
  G = c(rep(1, 180), rep(0, 320), rep(1, 40), rep(0, 60)),
  H = c(rep(1, 180), rep(0, 320), rep(1, 40), rep(0, 60))
))

set.seed(54321)
noise <- matrix(rbinom(prod(dim(temp)), 1, 0.05),
                ncol = ncol(temp))
meme <- 1*(temp | noise)
memberPlot(meme)
memberPlot(meme, ylab = "",
           features = c(123, 45, 18, 87, 75, 1234, 88, 999)) 

memberPlot <- function(bindat, features = NULL, pal = NULL,
                       xlab = "Members", ylab = "Categories", ...) {
  ## Ensure that it is really a binary matrix
  if(any(is.na(bindat))) stop("Data contains mssing values.")
  if(!all(bindat %in% 0:1)) stop("Matrix values must equal '0' or '1'.")
  if(is.null(pal)) {
    pal <- c("gray85",
             light.colors(24)[c(4, 3, 23, 9, 24,
                                7, 18, 11, 14, 5,
                                2, 15, 16, 21)],
             "purple3")
  }
  while(length(pal) < nrow(bindat)) pal <- c(pal, pal[-1])
  # rows are categories, columns are samples

  counts <- apply(bindat, 1, sum)  # number of members of each category
  M <- 1:nrow(bindat)
  bindat <- sweep(bindat, 1, M, "*")
  bindat <- bindat[rev(order(counts)),] # sort rows by membership
  if(!missing(features)) features <- features[rev(order(counts))]
  # order columns successively by row membership
  bd <- as.list(as.data.frame(t(bindat))) # temporary, for ordering
  oo <- do.call(order, bd)
  bindat <- bindat[, rev(oo)]
  ocount <- apply(bindat, 2, sum)
  bindat <- bindat[, ocount > 0]
  image(1:ncol(bindat), 1:nrow(bindat), t(bindat),
        col = pal[1:(1+nrow(bindat))],
        xlab = xlab, ylab = ylab, yaxt = "n", ...)
  if (!is.null(features)) {
    mtext(rownames(bindat), side =2, las = 2, at = 1:nrow(bindat)+0.1, line = 1)
    mtext(paste("D =", features), font = 2, side = 2, las = 2,
          at = 1:nrow(bindat)-0.2, line = 1)
  } else {
    mtext(rownames(bindat), side =2, las = 2, at = 1:nrow(bindat), line = 1)
  }
  text(ncol(bindat)/2, 1:nrow(bindat),
       paste("N =", apply(bindat > 0, 1, sum)),
       font = 2, cex=1.2)
  invisible(bindat)
}

temp <- read.table("temp.txt", sep="\t", comment.char='', as.is=TRUE)

foo <- strsplit(temp$V1, "\\s")
ping <- unlist(lapply(foo, function(f) which(!is.na(as.integer(f)))))
counter <- as.integer(sapply(1:length(foo), function(i) {
  foo[[i]][ping[i]]
}))
summary(diff(counter)) # all 1's

shortName <- unlist(sapply(1:length(foo), function(i) {
  paste(foo[[i]][1:(ping[i]-1)], collapse="_")
}))

pong <- unlist(lapply(foo, function(f) {
  w <- which(f %in% c('see', 'not'))
  ifelse(length(w) > 0, w-1, length(f))
}))

longName <- unlist(sapply(1:length(foo), function(i) {
  f <- foo[[i]]
  paste(f[(ping[i]+1):pong[i]], collapse="_")
}))
ln <- strsplit(longName, ',')
longName <- unlist(lapply(ln, function(x) x[1]))

iscc <- data.frame(shortName, longName, Hex=temp$V2)
write.table(iscc, file="iscc.txt", sep="\t", row.names=FALSE, quote=FALSE)
save(iscc, file="iscc.rda")

xkcd <- read.table("xkcdColors.txt", sep = "\t", skip = 1,
                   comment.char = "", quote = "", fill = TRUE)
xkcd <- xkcd[, 1:2]
colnames(xkcd) <- c("Name", "Hex")
ul <- gsub(" ", "_", xkcd$Name)
xkcd$Name <- ul
save(xkcd, file = "../../data/xkcd.rda")

# Copyright (C) Kevin R. Coombes, 2016

### Internal
xform <- function(colset, alpha=FALSE) {
  mat <- t(col2rgb(colset, alpha=alpha)/255)
  if (alpha) {
    value <- rgb(mat[, 1:3], alpha=mat[,4])
  } else {
    value <- rgb(mat[, 1:3])
  }
  if (is.null(names(colset))) {
    names(value) <- paste("X", 1:length(value), sep='')
  } else {
    names(value) <- names(colset)
  }
  value
}


invertColors <- function(...) {
  par(bg="gray30", fg="white",
      col.axis="white", col.lab="white",
      col.main="white", col.sub="white", ...)
}

rancurves <- function(colorset, ...) {
  colorset <- xform(colorset)
  plot(c(-2*pi, 2*pi), c(-5,5), type="n", xlab="Angle", ylab="Intensity", ...)
  x <- seq(-2*pi, 2*pi, length=500)
  L <- length(colorset)
  phase <- runif(L, 0, 2*pi)
  ampl <- rnorm(L, 0, 2)
  for (i in 1:L) {
    lines(x, ampl[i] * cos(phase[i] + x), col=colorset[i], lwd=2)
  }
  invisible(colorset)
}

ranpoints <- function(colorset, N=10, ...) {
  colorset <- xform(colorset)
  L <- length(colorset)
  mycols <- rep(colorset, each=N)
  x <- rep(rnorm(L, 0, 3), each=N) + rnorm(L*N)
  y <- rep(rnorm(L, 0, 3), each=N) + rnorm(L*N)
  plot(x, y, pch=16, col=mycols, ...)
  invisible(colorset)
}

uvscatter <- function(colorset, main=deparse(substitute(colorset)), ...) {
  cset <- xform(colorset)
  luvmat <- as(hex2RGB(cset), "LUV")
  x <- luvmat@coords
  plot(x[,2], x[,3], pch=16, col=cset, cex=2,
       xlab="U", ylab="V", main=main, ...)
  invisible(cset)
}

luminance <- function(colorset, main=deparse(substitute(colorset)), ...) {
  cset <- xform(colorset)
  luvmat <- as(hex2RGB(cset), "LUV")
  x <- luvmat@coords
  ox <- order(x[,1])
  plot(x[ox,1], pch=16, col=cset[ox], cex=2,
       ylab="Luminance", main=main, ...)
  invisible(cset)
}

plothc <- function(colorset, main=deparse(substitute(colorset)), ...) {
  cset <- xform(colorset)
  luvmat <- as(hex2RGB(cset), "LUV")
  x <- luvmat@coords
  euc <- dist(x, "euclid")
  hd <- hclust(euc, "ward.D2")
  plot(hd, hang = -1, labels = rep("", length(cset)), 
       main = main, ...)
  mtext(names(cset), side = 1, line = 0, at = order(hd$order), 
        col = cset, las = 2)
  invisible(hd)
}

plotpc <- function(colorset, main=deparse(substitute(colorset)), ...) {
  cset <- xform(colorset)
  luvmat <- as(hex2RGB(cset), "LUV")
  x <- luvmat@coords
  pc <- princomp(x)
  plot(pc$scores[,1], pc$scores[,2],
       pch=16, col=cset, cex=2,
       xlab="PC1", ylab="PC2", main=main, ...)
  invisible(pc)
}

# added to allow external use of rgl
getLUV <- function(colorset) {
  cset <- xform(colorset)
  luvmat <- as(hex2RGB(cset), "LUV")
  list(coords = luvmat@coords, cset = cset)
}

p3d <- function(colorset, main=deparse(substitute(colorset)), ...) {
  y <- getLUV(colorset)
  x <- y$coords
  cset <- y$cset
  scatterplot3d(x, color = cset, pch = 16, ...)
  invisible(cset)
}

swatch <- function(colorset, main=deparse(substitute(colorset))) {
  cset <- xform(colorset)
  luvmat <- as(hex2RGB(cset), "LUV")
  x <- luvmat@coords
  labelcols <- c("white", "black")[1 + 1*(x[,1] > 50)]
  L <- length(cset)
  pts <- barplot(rep(1, L), col=cset, main=main, yaxt='n')
  text(pts, 0.5, names(cset), srt=90, col=labelcols)
  invisible(pts)
}

sortByHue <- function(colorset) {
  cset <- xform(colorset)
  luv <- as(hex2RGB(cset), "HSV")
  tink <- luv@coords[,1]
  cset[order(tink)]
}

swatchHue <- function(colorset,
                        main=paste(deparse(substitute(colorset)),
                                   ", by Hue", sep="")) {
  hh <- sortByHue(colorset)
  swatch(hh, main=main)
}

sortByLuminance <- function(colorset) {
  cset <- xform(colorset)
  luv <- as(hex2RGB(cset), "LUV")
  tink <- luv@coords[,1]
  cset[order(tink)]
}

swatchLuminance <- function(colorset,
                        main=paste(deparse(substitute(colorset)),
                                   ", by Luminance", sep="")) {
  hh <- sortByLuminance(colorset)
  swatch(hh, main=main)
}

ranswatch  <- function(colorset,
                        main=deparse(substitute(colorset))) {
  cset <- xform(colorset)
  L <- length(cset)
  scramble <- sample(cset)
  swatch(scramble, main=main)
}

luvDistances <- function(colorset) {
  colorset <- xform(colorset)
  if (any(dup <- duplicated(colorset))) {
    reserved <- colorset[dup]
    colorset <- colorset[!dup]
  }
  # work in LUV space
  luvmat <- as(hex2RGB(colorset), "LUV")
  if(any(is.na(luvmat@coords))) {
    luvmat@coords[is.na(luvmat@coords)] <- 0
  }
  # take the first color as the starting point
  selected <- 1
  names(selected) <- names(colorset)[1]
  mind <- d2(selected, luvmat)
  dd <- NULL
  # loop through, finding the most well-seaprated color at each iteration
  for (i in 2:length(colorset)) {
    idx <- which(mind==max(mind))[1]
    dd <- c(dd, sqrt(mind[idx]))
    mind <- upd(idx, luvmat, mind)
    selected <- c(selected, idx)
  }
  dd <- c(max(dd), dd)
  names(dd)[1] <- names(colorset)[1]
  colors <- colorset[selected]
  if (any(dup)) {
    colors <- c(colors, reserved)
    extra <- rep(0, length(reserved))
    names(extra) <- names(reserved)
    dd <- c(dd, extra)
  }
  list(colors=colors, distances=dd)
}

computeDistances <- function(colorset) {
  luvDistances(colorset)$distances
}

plotDistances <- function(colorset, main=deparse(substitute(colorset)), pch=16, ...) {
  cset <- xform(colorset)
  luvd <- luvDistances(cset)
  dd <- luvd$distances
  plot(dd, main=main,
       xlab="Index", ylab="Distance in L*u*v* space",
       col=luvd$colors, pch=pch, ...)
  abline(h=40)
  invisible(luvd)
}

# Copyright (C) Kevin R. Coombes, 2016

colorNames <- function (colorset)  {
  colorset <- xform(colorset)
  # note: should only use 'col2rgb' for color() names
  rigby <- t(col2rgb(colors())) # R-G-B matrix of
  rownames(rigby) <- colors()   #  known color names
  csrgb <- sRGB(rigby/255)      # convert to colorpace
  csluv <- as(csrgb, "LUV")     # then to L*u*v*
  cc <- csluv@coords
  cc[is.na(cc)] <- 0            # handle 'black'
  d3 <- function(y0) {
    # find the closest match in L*u*v* space
    temp <- sweep(cc, 2, y0, "-")
    dist <- apply(temp^2, 1, sum)
    which(dist == min(dist))[1]
  }
  alpha <- as(hex2RGB(colorset), "LUV")
  m <- apply(alpha@coords, 1, d3)
  rownames(cc)[m]
}

isccNames <- function(colorset) {
  colorset <- xform(colorset)
  data("iscc", package="Polychrome", envir=environment())
  iscc <- get("iscc", envir=environment())
  munsell <- as(hex2RGB(iscc$Hex), "LUV")
  d3 <- function(y0) {
    temp <- sweep(munsell@coords, 2, y0, "-")
    dist <- apply(temp^2, 1, sum)
    which(dist==min(dist))
  }
  alpha <- as(hex2RGB(colorset), "LUV")
  m <- apply(alpha@coords, 1, d3)
  iscc$longName[m]
}

xkcdNames <- function(colorset) {
  colorset <- xform(colorset)
  data("xkcd", package="Polychrome", envir=environment())
  xkcd <- get("xkcd", envir=environment())
  munsell <- as(hex2RGB(xkcd$Hex), "LUV")
  d3 <- function(y0) {
    temp <- sweep(munsell@coords, 2, y0, "-")
    dist <- apply(temp^2, 1, sum)
    which(dist==min(dist))
  }
  alpha <- as(hex2RGB(colorset), "LUV")
  m <- apply(alpha@coords, 1, d3)
  xkcd$Name[m]
}

makePalette <- function(n, colorset) {
  if (n < 3) {
    warning("minimal value for n is 3, returning requested palette with 3 levels\n")
    n <- 3
  }
  L <-  length(colorset)
  if (n > L) {
    name <- deparse(substitute(colorset))
    warning(paste("n too large, allowed maximum for palette", 
                  name, "is", L), 
            "\nReturning the palette you asked for with that many colors\n")
    n <- L
  }
  colorset[1:n]
}
####################################
palette36.colors <- function(n = 36) {
  data("palette36", package="Polychrome", envir=environment())
  palette36 <- get("palette36", envir=environment())
  return (makePalette(n, palette36))
}

####################################
alphabet.colors <- function(n = 26) {
  data("alphabet", package="Polychrome", envir=environment())
  alphabet <- get("alphabet", envir=environment())
  return (makePalette(n, alphabet))
}

####################################
glasbey.colors <- function(n = 32) {
  data("glasbey", package="Polychrome", envir=environment())
  glasbey <- get("glasbey", envir=environment())
  return (makePalette(n, glasbey))
}

####################################
light.colors <- function(n = 24) {
  data("Light24", package="Polychrome", envir=environment())
  light <- get("Light24", envir=environment())
  return (makePalette(n, light))
}

####################################
dark.colors <- function(n = 24) {
  data("Dark24", package="Polychrome", envir=environment())
  dark <- get("Dark24", envir=environment())
  return (makePalette(n, dark))
}

####################################
sky.colors <- function(n = 24) {
  data("sky-colors", package="Polychrome", envir=environment())
  sky <- get("sky.colors", envir=environment())
  return (makePalette(n, sky))
}

####################################
# Kelly
kelly.colors <- function(n = 22) {
  kelly <- c(
    white = "#f2f3f4",
    black = "#222222",
    yellow = "#f3c300",
    purple = "#875692",
    orange = "#f38400",
    lightblue = "#a1caf1",
    red = "#be0032",
    buff = "#c2b280",
    gray = "#848482",
    green = "#008856",
    purplishpink = "#e68fac",
    blue = "#0067a5",
    yellowishpink = "#f99379",
    violet = "#604e97",
    orangeyellow = "#f6a600",
    purplishred = "#b3446c",
    greenishyellow = "#dcd300",
    reddishbrown = "#882d17",
    yellowgreen = "#8db600",
    yellowishbrown = "#654522",
    reddishorange = "#e25822",
    olivegreen = "#2b3d26"
  )
  return (makePalette(n, kelly))
}

####################################
# Green-Armytage

green.armytage.colors <- function(n=26) {
  colsch <- list(
    amethyst=c(240, 163, 255),
    blue=c(0, 117, 220),
    caramel=c(153,63,0),
    damson=c(76, 0, 92),
    ebony=c(25, 25, 25),
    forest=c(0, 92, 49),
    green=c(43, 206, 72),
    honeydew=c(255, 204, 153),
    iron=c(128, 128, 128),
    jade=c(148, 255, 181),
    khaki=c(143, 124, 0),
    lime=c(157, 204, 0),
    mallow=c(194, 0, 136),
    navy=c(0, 51, 128),
    orpiment=c(25, 164, 5),
    pink=c(255, 168, 187),
    quagmire=c(66, 102, 0),
    red=c(255, 0, 16),
    sky=c(94, 241, 242),
    turquoise=c(0, 153, 143),
    uranium=c(224, 255, 102),
    violet=c(16, 10, 255),
    wine=c(153, 0, 0),
    xanthin=c(255, 255, 128),
    yellow=c(255, 225, 0),
    zinnia=c(255, 80, 0))
  R <- unlist(lapply(colsch, function(x) x[1]/255))
  G <- unlist(lapply(colsch, function(x) x[2]/255))
  B <- unlist(lapply(colsch, function(x) x[3]/255))
  alpha <- as(sRGB(R, G, B), "LUV")
  green.armytage <- hex(alpha)
  names(green.armytage) <- names(colsch)
  makePalette(n, green.armytage)
}

# Copyright (C) Kevin R. Coombes, 2016

minOf2 <- function(a, b) (a + b - abs(a-b))/2

d1 <- function(y0, luv) {
  temp <- sweep(luv@coords, 2, y0, "-")
  apply(temp^2, 1, sum)
}

d2 <- function(idx, luv) {
  d1(luv@coords[idx,], luv)
}

upd <- function(idx, luv, mind) {
  temp <- d2(idx, luv)
  minOf2(mind, temp)
}

createPalette <- function(N, seedcolors, prefix="NC", range=c(30, 90),
                          target = c("normal", "protanope", "deuteranope", "tritanope"),
                          M=50000) {
  DARK <- min(range)
  LIGHT <- max(range)
  bigset <- as.matrix(data.frame(R=sample(0:255, M, replace=TRUE),
                                 G=sample(0:255, M, replace=TRUE),
                                 B=sample(0:255, M, replace=TRUE)))/255
  target <- match.arg(target)
  if (target != "normal") bigset <- colorDeficit(bigset, target)
  bigrgb <- RGB(bigset)
  bigluv <- as(bigrgb, "LUV")
  toodark <- bigluv@coords[,1] < DARK
  toolight <- bigluv@coords[,1] > LIGHT
  goldilocks <- !toodark & !toolight
  bigset <- bigset[goldilocks,]
  bigrgb <- RGB(bigset)
  bigluv <- as(bigrgb, "LUV")

  s <- as(hex2RGB(seedcolors[1]), "LUV")@coords
  seedcolors <- seedcolors[-1]
  distance <- d1(s, bigluv)
  selected <- which(distance == min(distance))
  mind <- d2(selected, bigluv)
  if (length(seedcolors) > 0) {
    repeat {
      s <- as(hex2RGB(seedcolors[1]), "LUV")@coords
      seedcolors <- seedcolors[-1]
      distance <- d1(s, bigluv)
      idx <- which(distance == min(distance))
      selected <- c(selected, idx)
      mind <- upd(idx, bigluv, mind)
      if (length(seedcolors) == 0) break
    }
  }
  for (i in (1+length(selected)):N) {
    idx <- which(mind == max(mind))[1]
    mind <- upd(idx, bigluv, mind)
    selected <- c(selected, idx)
  }
  nonsense <- bigluv[selected,]
  newcols <- hex(nonsense)
  names(newcols) <- paste(prefix, 1:N, sep="")
  newcols
}  

---
title: "Using Polychrome With ggplot"
author: "Kevin R. Coombes"
data: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using Polychrome With ggplot}
  %\VignetteKeywords{Polychrome,Color Palettes,Palettes,ggplot}
  %\VignetteDepends{Polychrome}
  %\VignettePackage{Polychrome}
  %\VignetteEngine{knitr::rmarkdown}
---

In this vignette, we describe how to use <tt>Polychrome</tt> palettes
with the package <tt>ggplot2</tt>. The vignette will only run code if
the <tt>ggplot2</tt> package is available.
```{r}
evalVignette <- requireNamespace("ggplot2", quietly = TRUE)
knitr::opts_chunk$set(eval = evalVignette)
```
## Getting Started
We want to build a custom palette of 40 colors for this application, with
each block of four consecutive colors being distinguishable. We start by
constructing a new palette in the usual way.

```{r newpal, fig.cap="A new palette of 40 colors.", fig.width = 8, fig.height=4}
library(Polychrome)
set.seed(935234)
P40 <- createPalette(40, c("#FF0000", "#00FF00", "#0000FF"), range = c(30, 80))
swatch(P40)
```
We achieve the goal of making the blocks of four colors being distinguishable
by first sorting by hue, and then rearranging them into four-blocks.
```{r reorder, fig.cap="A sorted palette of 40 colors.", fig.width = 8, fig.height=4}
P40 <- sortByHue(P40)
P40 <- as.vector(t(matrix(P40, ncol=4)))
swatch(P40)
```
Here is the **key point** of this entire vignette: By default,
<tt>Polychrome</tt> gives names to each of the colors in a palette.
But, in <tt>ggplot</tt>, named colors will only be applied if they
match the levels of an appropriate factor in the data. The simplest
solution is to remove the names:
```{r dename}
names(P40) <- NULL
```

## Simulating Complex Data
For illustration purposes, we simulate a data set with a moderately
complex structure. Specifically, we assume that we have

* Nine groups of samples,
* Four samples in each group,
* Three replicate experiments for each sample, and
* Measurements repeated over four days.

Here is the simulated design of the data set.
```{r simdata}
## Nine groups
NG <- 9
gp <- paste("G", 1:NG, sep = "")
length(gp)

## Four Subjects per group
## 36 Subjects = 9 groups * 4 subjects/group 
sid <- paste(rep(LETTERS[1:2], each=26), c(LETTERS, LETTERS), sep="")[1:(4*NG)]
length(sid)

## Three Reps per subject
## 108 Experiments
reps = factor(rep(c("R1", "R2", "R3"), times = length(sid)))
length(reps)

## Each experiment with measurements on four Days, so 432 data rows
daft <- data.frame(Day = rep(1:4, each=length(reps)),
                   Group = factor(rep(rep(gp, each=12), times = 4)),
                   Subject = factor(rep(rep(sid, each = 3), times=4)),
                   Rep = factor(rep(reps, times = 4)))
dim(daft)
summary(daft)
```

Now we add simulated "measurements" taken on each replicate of each subject
on each of four days.
```{r variable}
## Linear model with noise, ignoring group
beta <- runif(length(sid), 0.5, 2)
## "Measured" variable
attach(daft)
daft$variable <- rnorm(nrow(daft), 0, 0.2) + 1  + beta[as.numeric(Subject)]*Day
detach()
```

### Plotting the results.

```{r fig.cap="A faceted plot, colored by subject.", fig.width=8, fig.height=8}
library(ggplot2)
ggplot(daft, aes(x = Day, y = variable, colour = as.factor(Subject))) +
  geom_point(aes(shape = as.factor(Rep)), size = 3) +
  geom_line(aes(linetype = as.factor(Rep)), size = 0.8) +
  facet_wrap(. ~ Group, ncol = 3)+
  theme_bw() + theme(legend.position="none")+
  scale_color_manual(values = P40)
```

---
title: "Polychrome: Color Deficits"
author: "Kevin R. Coombes"
data: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Color Deficits}
  %\VignetteKeywords{OOMPA,Polychrome,Color Palettes,Palettes}
  %\VignetteDepends{Polychrome}
  %\VignettePackage{Polychrome}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r opts, echo=FALSE}
knitr::opts_chunk$set(fig.width=8, fig.height=5)
```
# Introduction

There are three main forms of color-deficient vision, depending on
which of the three types of cones (red, green, or blue) are missing
or defective in the eyes of an individual.

1. Deuteranopes (about 6% of men) are red-blind, and so they have
   trouble distinguishing red from green.
2. Protanopes (about 2% of men) are green-blind, and they also have
   trouble distinguishing red from green.
3. Tritanopes (less than 1% of men) are blue-blind, and they have
   trouble distinguishing blue from yellow.

<tt>Polychrome</tt> includes facilities to simulate different forms of
color deficit and to create palettes that can be used by individuals
with color deficient vision. This vignette describes those facilities.

# Red-Green Color Maps
We start by loading the package.
```{r pkgs}
library(Polychrome)
```
Next, we create the common "standard" color map used by 
bioinformaticians to draw heatmaps. 
```{r rg, fig.cap="Common red-green palette."}
rg <- colorRampPalette(c("red", "black", "green"))(64)
image(matrix(1:64), col=rg)
```

Now, we convert it to what would be seen by a deuteranope.
```{r deut, fig.cap="Deuteranopes' view of the common red-green palette."}
rg.deut <- colorDeficit(rg, "deut")
image(matrix(1:64), col=rg.deut)
```

We can also convert it to what would be seen by a protanope...
```{r prot, fig.cap="Protanopes' view of the common red-green palette."}
rg.prot <- colorDeficit(rg, "prot")
image(matrix(1:64), col=rg.prot)
```

... or a tritanope.
```{r trit, fig.cap="Tritanopes' view of the common red-green palette."}
rg.trit <- colorDeficit(rg, "trit")
image(matrix(1:64), col=rg.trit)
```

# Large Qualitative Palettes
We repeat those operations for the 26-color alphabet palette from
<tt>Polychrome</tt>. 

```{r alfa, fig.cap = "The 26-color alphabet palette."}
alfa <- alphabet.colors(26)
swatch(alfa)
```

```{r adeut, fig.cap = "The 26-color alphabet palette, as seen by a deuteranope."}
alfa.deut <- colorDeficit(alfa, "deut")
swatch(alfa.deut)
```

```{r aprot, fig.cap = "The 26-color alphabet palette, as seen by a protanope."}
alfa.prot <- colorDeficit(alfa, "prot")
swatch(alfa.prot)
```

```{r atrit, fig.cap = "The 26-color alphabet palette, as seen by a tritanope."}
alfa.trit <- colorDeficit(alfa, "trit")
swatch(alfa.trit)
```

# Creating New Palettes
We can also use the <tt>createPalette</tt> function to create new palettes
designed for individuals with color deficits. This operation requires us to
use the <tt>target</tt> option of the function. The default value of "normal"
assumes that the target audience consists of individuals with normal color
vision.

```{r cp1, fig.cap="A palette designed for deuteranopes."}
cp <- createPalette(20, "#111111", target="deuteranope")
names(cp) <- colorNames(cp)
swatch(cp)
```


```{r cp2, fig.cap = "A palette designed for protanopes."}
cp <- createPalette(20, "#111111", target="protanope")
names(cp) <- colorNames(cp)
swatch(cp)
```


```{r cp3, fig.cap = "A palette designed for tritanopes."}
cp <- createPalette(20, "#111111", target="tritanope")
names(cp) <- colorNames(cp)
swatch(cp)
```


## A Small Color-Safe Palette

Here we try to build a small palette that has at least some chance of being
useful for people with any of the three forms of color-blindness. First, we
get a slightly larger starting palette, and convert it to simulate each of
the three forms of color deficits.
```{r dists}
p34 <- palette36.colors(36)[3:36]
names(p34) <- colorNames(p34)
p34.deut <- colorDeficit(p34, "deut")
p34.prot <- colorDeficit(p34, "prot")
p34.trit <- colorDeficit(p34, "trit")
```

Next, we reorder the palettes by distinguishability.
```{r shift}
shift <- function(i, k=34) c(i, 1:(i-1), (1+i):k)
co <- shift(13)
pd <- computeDistances(p34.deut[co])
pp <- computeDistances(p34.prot[co])
pt <- computeDistances(p34.trit[co])
```

Now we use the ranks from this ordering to compute a distinguishability score.
```{r score}
rd <- rank(pd)[order(names(pd))]
rp <- rank(pd)[order(names(pp))]
rt <- rank(pd)[order(names(pt))]
score <- 2*rd + 1.5*rp + rt
```

Now we show the top ten colors (ranked by this score) , adjusting for different
color deficits.
```{r, fig.width=8, fig.height=7, fig.cap="Color safe palette."}
x <- p34[names(rev(sort(score)))][1:10]
y <- colorDeficit(x, "deut")
z <- colorDeficit(x, "prot")
w <- colorDeficit(x, "trit")

opar <- par(mfrow=c(2,2))
swatch(x, main="Normal")
swatch(y, main="Deuteranope")
swatch(z, main="Protanope")
swatch(w, main="Tritanope")
```

```{r, echo=FALSE}
par(opar)
```

# Conclusion
We have illustrated how to convert arbitrary palettes into versions that
simulate different forms of color deficits. We have also shown how to
create palettes specifically for individuals with different color deficits.
Finally, we have created a ten-color palette that has a reasonable chance
of working simultaneously for individuals with any of the three most common
form of color deficits.

---
title: "Polychrome: Plots of Many Colors"
author: "Kevin R. Coombes"
data: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Polychrome}
  %\VignetteKeywords{OOMPA,Polychrome,Color Palettes,Palettes}
  %\VignetteDepends{Polychrome}
  %\VignettePackage{Polychrome}
  %\VignetteEngine{knitr::rmarkdown}
---

In this vignette, we describe how to use the palettes that ship with
the <tt>Polychrome</tt> package. We also describe the tools that are
provided to view palettes in different settings and different kinds
of plots.

## Getting Started

As usual, we start by loading the package:

```{r}
library(Polychrome)
```

Next, we can look at one of the color palettes it provides.

```{r}
mypal <- kelly.colors(22)
```

```{r fig.width=7, fig.height=5}
swatch(mypal)
```

## Resorted barplots

To get a better idea of whether colors are distinguishable, we can
sort them by hue:
```{r fig.width=7, fig.height=5}
swatchHue(mypal)
```

or by luminance:
```{r fig.width=7, fig.height=5}
swatchLuminance(mypal)
```

or in random order:
```{r fig.width=7, fig.height=5}
ranswatch(mypal)
```

## Scatter Plots
Some colors that work well in barplots may work less well in scatter
plots.  To test that possibility, we can generate some random points
and plot them. The next line of code generates 14 points from each
color in the "alphabet" palette.

```{r fig.width=7, fig.height=5}
pal2 <- alphabet.colors(26)
ranpoints(pal2, 14)
```

Our standard way to think about colors is in the L\*u\*v\* color space
model defined by the CIE.  The next plot shows how the points in the
palette are distributed in the u-v subspace.
```{r fig.width=7, fig.height=5}
uvscatter(pal2)
```

We can also show the associated luminance, L.
```{r fig.width=7, fig.height=5}
luminance(pal2)
```

## Plotting Curves
We can also test if a palette is useful for drawing a graph full of
different curves rarther than points.
```{r fig.width=7, fig.height=5}
rancurves(pal2)
```

## Color Similarity
We use L\*u\*v\* color space because perceptual distinguishability seems
to be closely related to Euclidean distance in this space.  To get
another view of which colors are most similar, we can perform
hierarchical clustering of colors using this representation.
```{r fig.width=7, fig.height=5}
plothc(pal2)
```

We can also perform principal components in L\*u\*v\* space to display
a projection that tries to preserve distances.
```{r fig.width=7, fig.height=5}
plotpc(pal2)
```

## Grayscale
One further consideration that should be kept in mind when creating
color palettes is that not every potential viewer has normal color
vision. One way to test if a color palette will remain useful in this
context is to convert it to gray scale. We do this by converting to
grayscale using the desaturate function from the colorspace package,
and then preparing a barplot by luminance. 

```{r fig.width=7, fig.height=5}
grayed <- colorspace::desaturate(pal2)
swatchLuminance(grayed)
```

---
title: "Polychrome: Creating New Palettes"
author: "Kevin R. Coombes"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Creating Palettes with Polychrome}
  %\VignetteKeywords{OOMPA,Polychrome,Color Palettes,Palettes}
  %\VignetteDepends{Polychrome}
  %\VignettePackage{Polychrome}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r opts,echo=FALSE}
knitr::opts_chunk$set(fig.width=7, fig.height=5)
```

Although the <tt>Polychrome</tt> package ships with several pre-built
multicolored palettes, it also contains tools to build more.  In this
vignette, we expain how to use the <tt>createPalette</tt> function for
this purpose.

# Getting Started

As usual, we start by loading the package:

```{r lib}
library(Polychrome)
```

Here are the arguments of the main function.
```{r args}
args(createPalette)
```

+ <tt>N</tt> is the number of colors to include in the palette
+ <tt>seedcolors</tt> is a list of one or more starting colors that you would
like included in the palette.
+ <tt>prefix</tt> is a character string used to create default names for the
palette colors.
+ <tt>range</tt> is a numeric vector of length 2 indicating the minimum and
maximum desired luminance for the palette.
+ <tt>M</tt> is an an integer representing the number of random colors to be
generated in order to create the palette.

## Recommendations

1. The seed colors usually have a small effect on the overall palette,
but they can change the order.  White and black appear to be
exceptions; they tend not to be included in the palette unless forced.
One reason to use more seed colors would be to aumgent an existing
small palette by adding distinguishable colors.
2. We show below that an upper bound on a good set of colors is likely to be
around 40. It may be better in the long run to build a palette that you like
of about this size, and then just use colors from the beginning if you don't
need that many.
3. The speed of the algorithm mainly depends on <tt>M</tt>. You get a richer
selection of colors with larger <tt>M</tt>. We have been successful using the
default of 50000.
4. The range depends on whether you expect to use a light background (take 
<tt>range = c(10, 60)</tt>), a dark background (take <tt>range = c(55,
95)</tt> )or both (take <tt>range = c(30, 80)</tt>).
5. Unless you really need colors called X1, ..., X37, you are probably better
off using the <tt>colorNames</tt> function to assign meaningful names after
creating the palette.

# A Small Palette
We'll start with a simple (but fast) example.
```{r first10}
set.seed(723451) # for reproducibility
firstpal <- createPalette(10, c("#010101", "#ff0000"), M=1000)
firstpal
```

The probably surprising result is that the seed colors are actually
_not_ included in the final palette. There are two reasons for this.
First, the colors 'black' and 'red' are outside the specified
(default) luminance range. So, they are replaced by the closest
randomly generated colors that are inside the range.  But that leads
us to the second reason. We specified a small value of <tt>M</tt>.
The algorithm operates by randomly selecting a set of potential colors
in the desired range, and will eventually assemble the palette from
this set.  With only 1000 candidates, there may not be anything that
looks close to pure red or pure black in the set of candidates.

```{r firstswatch, fig.cap="First palette"}
swatch(firstpal)
```

## Do the seeds matter?
Here is the palette that we get if we just start with the color black. Note that
we keep resetting the random seed -- the one for the random number generator --
to make sure we are choosing from the same set of randomly generated candidates.
```{r second, fig.cap="A pallete from a single black seed."}
set.seed(723451) # for reproducibility
second <- createPalette(10, c("#010101"), M=1000)
swatch(second)
```

This palette is essentially indistinguishable from the first one. However, here is
what happens if we just start with the color red.
```{r third, fig.cap="A palette from a single red seed."}
set.seed(723451) # for reproducibility
third <- createPalette(10, c("#ff0000"), M=1000)
swatch(third)
```

The red, green, and blue triumvirate stays near the top of the list, but black has
disappeared. What happens if we instead start with green and blue?
```{r fourth, fig.cap="A blue and green seeded palette."}
set.seed(723451) # for reproducibility
fourth <- createPalette(10, c("#00ff00", "#0000ff"), M=1000)
swatch(fourth)
```

Except for the forced reordering at the start, we get nearly the same palette.
So, let's instead try starting with cyan, magenta, and yellow.
```{r fifth, fig.cap="A CMY seeded palete."}
set.seed(723451) # for reproducibility
fifth <- createPalette(10, c("#00ffff", "#ff00ff", "#ffff00"), M=1000)
swatch(fifth)
```

We see that the initial seed colors have a relatively small effect.  This result
should not be unexpected based on the "greedy" nature of the underlying algorithm,
which tries at each step to find the color that is most distant from everything
already included in the list.  The vertices of the polyhedron in three-dimensional
LUV space include the usual primary colors.

# How many good colors can we find?
Now we'd like to examine how big we think we can realistically make <tt>N</tt>,
the number of distinguishable colors. We can estimate this by first estimating
the volume  of the visible region of LUV space. We can start by determining the
convex hull of the UV projection.  We saw in the manuscript that our 36-color
palette uses almost all of the visible region. So (for speed as well as
simplicity), we are going to work with that palette.

First, we have to get the LUV coordinates of the colors.
```{r luv36}
library(colorspace)
p36 <- palette36.colors(36)
luvmat <- as(hex2RGB(p36), "LUV")
y <- luvmat@coords
```
Now we can use a function from the <tt>grDevices</tt> package to compute the
convex hull.
```{r hull, fig.width=6,fig.height=6,fig.cap="The convex hull of the UV coordinates of the 36-color palette."}
hull <- y[chull(y[,2], y[,3]), 2:3]
uvscatter(p36)
polygon(hull[,1], hull[,2], lwd=2)
```

The next step is to compute the area of the convex hull. While there are R
packages that include functions to do this computation, the algorithm is
sufficiently easy that it hardly seems worth it to load another package.
```{r area}
hullaug <- rbind(hull, hull[1,])
N <- nrow(hullaug)
fst <-sum(hullaug[1:(N-1), 1]*hullaug[2:N, 2] )
snd <- sum(hullaug[2:N, 1]*hullaug[1:(N-1), 2] )
area <- (snd - fst)/2
area
```
So, a crude (over)estimate of the volume of the viewable region of LUV space is
to take the range of L values (0--100) and multiple it times the UV area. This
is an overestimate for two reasons. First, we can't actually use the entire
viewable range of luminances agains any single background, so the factor of 100
is probably closer to 60. Second, not every combination of L-U-V values in this
region designates an actual color.
```{r uppervol}
overvol <- 100*area
overvol
```

Now, we want colors to be 40 units apart to be reliably distinguishable by most
viewers. So, each time we pick a color, it should "occupy" a sphere of radius 20
to avoid picking indistinguishable colors.  The volume of such a sphere is:
```{r volsphere}
sphere <- 4/3*pi*20^3
sphere
```

To get the most generous (over)estimate of the number of colors, we pretend for
a moment that we can pack spheres into the space with minimal waste. By the
Kepler conjecture, we know that this means that the spheres can occupy about
74% of the volume.
```{r fat}
100 * area/sphere *0.74

60 * area/sphere *0.74
```
If we could use the full range of luminances, we could get about 82 colors. If
we can only use a range of luminance values about 60 units wide, then we can at
most hope to get 50 distinct colors.

### Improving the estimate
We can empirically improve our estimate of the volume of the visible portion of
LUV space.  We start by generating a tight, regularly spaced grid of values.
```{r what}
L <- 0:100
U <- -60:150
V <- -150:150
luv <- cbind(L = rep(L, length(U)*length(V)),
             U = rep(rep(U, each=length(L)), times=length(V)),
             V = rep(V, each=length(L)*length(U)))*1.0
```
Next, we convert the LUV coordinates into hexadecimal representations in RGB space.
```{r hexify}
tested <- LUV(luv)
co <- tested@coords
```
Whenever the conversion fails (yielding an <tt>NA</tt> value), the LUV coordinates
were not visible. So, we can estimate the fraction of the regular grid that is
visible:
```{r }
he <- hex(tested)
summary(is.na(he))
mean(!is.na(he))
```
That is roughly 20%.  Now we look at how many spheres we could pack into this
space.
```{r}
VOL <- diff(range(L)) * diff(range(U)) * diff(range(V)) * 0.205
VOL
VOL/sphere * 0.74
```
Hmm. This computation suggest that we can only find about 28 distinguishable
colors. Since we have examples of palettes with more colors than this, we must
now somehow be _underestimating_ the value.

But it's clear why.  Many of the spheres can be placed on the boundary, that is,
along the faces, edges, or even vertices of the polyhedron.  In that case, half
or more of their volume sticks outsde the polyhedron. If we crudely estimate
that half of the volume of half of the spheres can be ignored, then this
adjustment roughly undoes the correction for the sphere packing density. 
```{r final}
VOL/sphere
```
So, our final estimate is that the maximum number of distinguishable colors is 
roughly 38 or 39.  Being generous (and tending to favor round numbers), we're
going to take it to be about 40.

# Computational Complexity

The final parameter we want to look at is <tt>M</tt>, the number of candidate
colors that we search through. In general, we expect the time needed to run the
algorithm to be approximately <tt>N*M</tt>, since we must search through all
<tt>M</tt> candidates for each of the <tt>N</tt> desired colors. The trade-off,
of course, is that we can find more colors that are distinguishable with more
candidates. Here we increase <tt>M</tt> from 1000 to 10000.

```{r last10, fig.cap="Palette with more candidate colors."}
set.seed(723451) # for reproducibility
lastpal <- createPalette(10, c("#010101", "#ff0000"), M=10000)
lastpal
swatch(firstpal)
swatch(lastpal)
```

---
title: "Color Names"
author: "Kevin R. Coombes"
data: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{color names}
  %\VignetteKeywords{OOMPA,Polychrome,Color Palettes,Color names}
  %\VignetteDepends{Polychrome}
  %\VignettePackage{Polychrome}
  %\VignetteEngine{knitr::rmarkdown}
---

In this vignette, we describe various tools for assigning color names to the
hexadecimal representations of RGB colors.

## Getting Started

As usual, we start by loading the package:

```{r}
library(Polychrome)
```

Next, we can look at one of the color palettes it provides.

```{r}
data(alphabet)
```

```{r fig.width=7, fig.height=5}
swatch(alphabet)
```

As you can see, the `alphabet` palette has assigned names to colors 
that begin with different letters of the English alphabet. Since the
`Polychrome` pacakge includes three differnt sets of (more or less) 
standard color names, we can see what they do for these colors:

```{r manycolors}
cn <- colorNames(alphabet) # from UNIX rgb.txt file
cc <- isccNames(alphabet) # standards from the Inter-Society Color Council
xk <- xkcdNames(alphabet) # from the  xkCd online color survey
df <- data.frame(UNIX = cn, ISCC = cc, XKCD = xk)
rownames(df) <- names(alphabet)
df
```
Careful examination of this table shows that several ISCC names are duplicated:
```{r dups}
apply(df, 2, function(X) length(unique(X)))
dupn <- df$ISCC[duplicated(df$ISCC)]
tmp <- df[df$ISCC %in% dupn,]
tmp[order(tmp[,2]),]
```

## Scatter Plots
Our standard way to think about colors is in the L\*u\*v\* color space
model defined by the CIE.  The next plot shows how the points in various 
"color name spaces" are distributed in the u-v subspace.

```{r fig.width=12, fig.height=12}
data(xkcd)
data(iscc)
colMat <- col2rgb(colors())
opar <- par(mfrow = c(2,2))
unix <- rgb(red = colMat[1, ]/255, green = colMat[2, ]/255, blue = colMat[3, ]/255)
uvscatter(alphabet, xlim = c(-80, 160), ylim =c (-130, 125))
uvscatter(iscc$Hex, xlim = c(-80, 160), ylim =c (-130, 125), main = "ISCC")
uvscatter(unix, xlim = c(-80, 160), ylim =c (-130, 125), main = "UNIX")
uvscatter(xkcd$Hex, xlim = c(-80, 160), ylim =c (-130, 125), main = "XKCD")
par(opar)
```

As you can see, the `alphabet` colors are widely dispersed across the color
spectrum, making them somewhat easier to distinguish. The 247 standard colors
from the ISCC are clumped twoard the middle of the space, making it harder to
name colors out on the fringes. The UNIX rgb.txt color names consist of 697 
names, but some of these are duplicate names for the same 502 distinct 
hexadecimal color representations. Even so, there are clear gaps in the color 
space that are not given distinct names. The results fo teh XKCD survey have 
a much denser coverage of more of the available color space.

\name{distances}
\alias{distances}
\alias{plotDistances}
\alias{computeDistances}

\title{Visualizing Color Palettes}
\description{
  Functions that provide visualization of palettes to help determine
  appropriate contexts where thay can be used. 
}
\usage{
computeDistances(colorset)
plotDistances(colorset, main=deparse(substitute(colorset)), pch=16, ...)
}
\arguments{
  \item{colorset}{a character vector containing hexadecimal color
    values.}
  \item{main}{a character string, the main title for a plot}
  \item{pch}{Plotting character to use.}
  \item{\dots}{additional graphical parameters.}
}
\details{
  Carter and Carter established the fact that, for two colors to be
  reliably distinguished, the Euclidean distance between their
  representations in CIE L*u*v* color space should be at least 40
  units. The  \code{computeDistances} function reorders the colors by
  maximal separation in L\*u\*v\*  space, and computes the minimum
  distance of the next color to all the preceeding colors.  The
  \code{plotDistances} function computes distances and immediately plots
  the result.
}
\value{
  The \code{plotDistances} function returns a list with two vector
  components: the \code{colors} in sorted order, and the minimum
  \code{distances} from  each color to the set of preceeding colors. The
  \code{computeDistances} function returns the vector of minimum
  distances.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\references{
  Carter RC, Carter EC. High-contrast sets of colors. Applied Optics,
  1982; 21(16):2936--9.
  
  Coombes KR, Brock G, Abrams ZB, Abruzzo LV. Polychrome: Creating and
  Assessing Qualitative Palettes with Many Colors. Journal of
  Statistical Software. 2019; 90(1):1--23.

}
\seealso{
  \code{\link{palette.viewers}}
  }
\examples{
data(alphabet)
plotDistances(alphabet)
luvd <- computeDistances(alphabet)
}
\keyword{color}

\name{memberPlot}
\alias{memberPlot}

\title{Membership Plots}
\description{
  Membership plots are a graphical visualization of cross-membership of
  many individuals in many categories. Although Venn diagrams work well
  for this purpose when there are three or four categories, they can be
  difficiult to interpet, or even impossible to draw, with more
  categories. In this case, membership plots are more useful.
}
\usage{
memberPlot(bindat, features = NULL, pal = NULL, xlab = "Members",
           ylab = "Categories", \dots)
}
\arguments{
  \item{bindat}{A binary matrix where rows are categories, columns are
    members, 1 debnotes membership and 0 denotes nonmembership. Muissing
    data is not permitted.}
  \item{pal}{A character vector of colors. The first color is used for
    non-members; all other colors are used to denote different categories.}
  \item{features}{A numeric vector listing the number of features
    measured in each membership data set.}
  \item{xlab}{The usual graphical parameter.}
  \item{ylab}{The usual graphical parameter.}
  \item{\dots}{Additional plot parameters, especially "main" or "sub".}
}
\details{
  Membership plots are implemented as an image, where each row
  represents a different category and is shown in a different
  color. Non-membership is indicated by the same color regardless fo
  category; by dafault, we use "gray" for non-members. The data are
  sorted so that the number of members per category decreases from the
  bottom tot he top of the plot. They are also sorted so that membership
  in larger categories is prioritized from left to right.
}
\value{
  The transformed input matrix is returned invisibly. The main purpose
  of the function is teh "side-effect" or producing a plot.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\examples{
set.seed(98765)
categ <- 6
member <- 500
M <- matrix(rbinom(categ*member, 1, 0.5), nrow = categ)
rownames(M) <- LETTERS[1:categ]
memberPlot(M)
}
\keyword{color}

\name{iscc}
\alias{iscc}
\docType{data}
\title{Color Names From the Inter-Society Color Council (ISCC)}
\description{
  A data frame mapping hex codes for 267 colors to their official
  ISCC-NBS names.
}
\usage{data(iscc)}
\format{
  A data frame with three columns and 267 rows.
}
\details{
  This data set contains short names, long names, and hex codes for the
  267 official color namkes defineed by the ISCC. Data was obtained from
  the Texas Precancel CLub and reformatted to be used conveniently in R.
}
\source{
  Our main source was the no-longer-extant web site of the Texas
  Precancel Club ({http://tx4.us/nbs-iscc.htm}).
}
\references{
  See the Inter-Society Color Council web site
  (\url{https://iscc.org/}); the Wikipedia article on the ISCC-NBS
  system of color designation
  (\url{https://en.wikipedia.org/wiki/ISCC\%E2\%80\%93NBS_system}. 
}
\seealso{
  \code{\link{isccNames}}
}
\examples{
data(iscc)
head(iscc)
}
\keyword{datasets}
\keyword{color}

\name{invertColors}
\alias{invertColors}

\title{Inverting the Plot Device Color Scheme}
\description{
  Function to convert the default plot color scheme to white-on-black.
}
\usage{
invertColors(...)
}
\arguments{
  \item{\dots}{Other graphical parameters to be given to \code{par}.}
}
\details{
  This function changes the default color scheme of the current graphics
  device to white on black. Note that since \code{invertColors} resets
  the \code{bg} parameter, you should avoid passing in a new default
  value for the \code{col} parameter.
}
\value{
  It returns the original color scheme, which can be passed to the
  \code{par} command to restore the original values.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\seealso{
  \code{\link[graphics]{par}}
}
\examples{
opar <- invertColors()
plot(1:3, 4:6, pch=16)
par(opar)
}
\keyword{color}

\name{glasbey}
\alias{glasbey}
\docType{data}
\title{The 32-color Glasbye palette}
\description{
  A palette composed of 32 distinct colors.
}
\usage{data(glasbey)}
\format{
  A character string of length 32.
}
\details{
  A character vector containing hexadecimal color representations of 32
  distinctive colors that are well separated in the CIE L*u*v* color
  space.
}
\source{
  The color palette was created, using standard tools in the
  \code{colorspace} package from a manually transcribed matrix of
  RGB values copied from the paper by Glasbey and colleagues. 
}
\references{
  Glasbey CA, van der Heijden GWAM, Toh VFK, Gray AJ (2007). Colour Displays for
  Categorical Images. Color Research and Application, 32, 304-9.
}
\examples{
data(glasbey)
head(glasbey)
}
\keyword{datasets}
\keyword{color}

\name{palette.viewers}
\alias{palette.viewers}
\alias{Palette Viewers}
\alias{rancurves}
\alias{ranpoints}
\alias{swatch}
\alias{swatchHue}
\alias{swatchLuminance}
\alias{ranswatch}
\alias{uvscatter}
\alias{luminance}
\alias{plothc}
\alias{plotpc}
\alias{p3d}

\title{Visualizing Color Palettes}
\description{
  Functions that provide visualization of palettes to help determine
  appropriate contexts where thay can be used. 
}
\usage{
rancurves(colorset, ...)
ranpoints(colorset, N=10, ...)
swatch(colorset, main=deparse(substitute(colorset)))
swatchHue(colorset, main=paste(deparse(substitute(colorset)),
                           ", by Hue", sep=""))
swatchLuminance(colorset, main=paste(deparse(substitute(colorset)),
                           ", by Luminance", sep=""))
ranswatch(colorset, main=deparse(substitute(colorset)))
uvscatter(colorset, main=deparse(substitute(colorset)), ...)
luminance(colorset, main=deparse(substitute(colorset)), ...)
plothc(colorset, main=deparse(substitute(colorset)), ...)
plotpc(colorset, main=deparse(substitute(colorset)), ...)
p3d(colorset, main=deparse(substitute(colorset)), ...)
}
\arguments{
  \item{colorset}{a character vector containing hexadecimal color
    values.}
  \item{main}{a character string, the main title for a plot}
  \item{N}{an integer; the number of points to plot in each color.}
  \item{\dots}{additional graphical parameters.}
}
\details{
  Different palettes are useful in different contexts.  For example,
  high luminance colors may work well in barplots but provide low
  contrast when used to color points in scatter plots. The best way to
  decide if a palette is right for any particular application is
  probably to create a sample plot using the palette.  The functions
  described here provide sample plots that display colors.

  The function \code{rancurves} produces a set of sine curves with
  different phases and amplitudes, with each curve shown in a different
  color. The function \code{ranpts} produces a scatter plot showing
  \code{N} clustered points in each of the palette colors.

  There are four functions that use barplots to display the palette. The
  simplest one, \code{swatch}, simply produces one bar of height one for
  each color, in the order that they are listed in the palette.  The
  next two, \code{swatchHue} and \code{swatchLuminance}, first sort the palette
  (by hue or by luminance, respectively), before producing the
  barplot.  The goal of these functions is to make sure that similar
  colors can be distinguished by placing them close together. The final
  function, \code{ranswatch}, randomly sorts the colors, to help decide if
  similar colors are identifiable when they are relatively far apart.

  The \code{p3d} function plots the palette colors as spheres in
  three-dimensional CIE L*u*v* color space.  It has been shown that
  perceptual distance is closely related to Euclidean distance in L*u*v*
  space. The \code{uvscatter} function produces a scatter plot of the
  palette colors using their projected u-v coordinates. The
  \code{luminance} function sorts the colors by luminance and produces a
  scatter plot showing the luminance.

  The \code{plothc} function performs hierarchical clustering on the
  colors (using Euclidan distance in CIE L*u*v* color space and Ward's
  linkage) and displays the resulting dendrogram. The \code{plotpc}
  function uses the same distance metric to compute and plot principal
  components.

}
\value{
  In general, these functions are used for their side-effect (producing
  plots) rather than for their return values. In most cases, they
  invisibly return the color set with which they were invoked.  The
  barplot-based functions (\code{swatch}, \code{ranswatch}, \code{swatchHue}, and
  \code{swatchLuminance}), however, return the vector of bar-centers, which
  can be used to add other information to the plot. The \code{plothc}
  function returns the dendrogram, and the \code{plotpc} function
  returns the principal components object.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\seealso{
  \code{\link{palette36.colors}}
  }
\examples{
data(alphabet)
rancurves(alphabet)
ranpoints(alphabet)
uvscatter(alphabet)
luminance(alphabet)
plothc(alphabet)
p3d(alphabet, cex.symbols = 2)
swatch(alphabet)
swatchHue(alphabet)
swatchLuminance(alphabet)
ranswatch(alphabet)
}
\keyword{color}

\name{colorDeficit}
\alias{colorDeficit}

\title{Converting Colors to Illustrate Color Deficient Vision}
\description{
  Function to convert any palette to one that illustrates how it would
  appear to a person with a color deficit.
}
\usage{
colorDeficit(rgb, target = c("deuteranope", "protanope", "tritanope"))
}
\arguments{
  \item{rgb}{A color palette. Accepts hexademical representations, sRGB
class objects from the \code{colorspace} package, or three-column
sRGB matrices.}
  \item{target}{The kind of color deficit to simulate.}
}
\details{
  This function converts normal-vision color palettes into simulations
  that represent what is likely to be seen with one of the three kinds
  of color deficits.  Deuteranopes are red-blind, which is the most
common form of color deficit leading to an inability to distinguish red
and green. Protanopes are green-blind; this is the second most common
form of color-blindness and also leads to an inability to distinguish
red and green. Tritanopes are blue blind; this is the rarest form of
color blindness and leads to an inability to distinguish blue and yellow. 
}
\value{
  Returns a color palette in the same form as its input argument.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\seealso{
  \code{\link[colorspace]{color-class}}
}
\references{
  [1] http://www.vischeck.com/

  [2] Brettel H, Vienot F, Mollon JD. Computerized simulation of color appearance
for dichromats. J Opt Soc Am A Opt Image Sci Vis. 1997 Oct;14(10):2647-55. PubMed
PMID: 9316278.

  [3] Vienot F, Brettel H, Ott L, Ben M\'Barek A, Mollon JD. What do colour-blind
people see? Nature. 1995 Jul 13;376(6536):127-8. PubMed PMID: 7603561.
}
\examples{
alfa <- alphabet.colors(26)
def <- colorDeficit(alfa)
swatch(def)
}
\keyword{color}

\name{Dark24}
\alias{Dark24}
\alias{Light24}
\docType{data}
\title{Light and Dark 24-Color Palettes}
\description{
  Two palettes, each composed of 24 distinctive colors, optimized for
  either a light background (Dark24) or a dark background (Light24). 
}
\usage{
data(Dark24)
data(Light24)
}
\format{
  A character vector of length 24.
}
\details{
  A character vector containing hexadecimal color representations of 24
  distinctive colors that are well separated in the CIE L*u*v* color
  space.
}
\source{
  Both color palettes were generated using the
  \code{\link{createPalette}} function. In addition to specifing seed
  colors, the luminance range was restricted to produce either only
  light colors or only dark colors.
}
\seealso{
  \code{\link{createPalette}}
}
\examples{
data(Dark24)
Dark24
data(Light24)
swatch(Light24)
}
\keyword{datasets}
\keyword{color}

\name{alphabet}
\alias{alphabet}
\docType{data}
\title{A 26-Color Palette}
\description{
  A palette composed of 26 distinctive colors with names corresponding
  to letters of the alphabet.  
}
\usage{data(alphabet)}
\format{
  A character string of length 26.
}
\details{
  A character vector containing hexadecimal color representations of 26
  distinctive colors that are well separated in the CIE L*u*v* color
  space.
}
\source{
  The color palette was generated using the \code{\link{createPalette}}
  function with three seed colors: ebony ("#5A5156"), iron ("#E4E1E3"),
  and red ("#F6222E"). The colors were then manually assigned names
  begining with different letters of the English alphabet.
}
\seealso{
  \code{\link{createPalette}}
}
\examples{
data(alphabet)
alphabet
}
\keyword{datasets}
\keyword{color}

\name{colorsafe}
\alias{colorsafe}
\docType{data}
\title{A 10-Color Palette Distinguishable By COlor-Deficit Individuals}
\description{
  A palette composed of 10 distinctive colors, selected to be
  distinguishable by a mojority of individuals who have some form mof
  color deficient vision.
}
\usage{data(colorsafe)}
\format{
  A character string of length 10.
}
\details{
  A character vector containing hexadecimal color representations of 10
  distinctive colors that are well separated in the CIE L*u*v* color
  space, chosen to be distinguishable by people with color deficient vision.
}
\source{
  Details of how te paklette was constructed can be found in the
  "color-deficit" vignette.
}
\seealso{
  \code{\link{createPalette}}
}
\examples{
data(colorsafe)
colorsafe
\dontrun{vignette("color-deficits")}
}
\keyword{datasets}
\keyword{color}

\name{getLUV}
\alias{getLUV}

\title{Get the L*u*v* coordinates of the colors}
\description{
  Given a character vector of colors in any format acceptable to R, this
  function computes their coordinates in L*u*v* color space.
}
\usage{
getLUV(colorset)
}
\arguments{
  \item{colorset}{a vector containing color values in any format
    recognized by R.}
}
\details{
  The real point of this function is to allow users to plot the colors
  in three-dimensional space using the \code{rgl} package. Because
  \code{rgl} depends on an external installation of XQuartz on Macintosh
  computers (and because we have found that some students in courses
  that we teach are apparently unable to install XQuartz, especially if
  they are using institutional computers without adminstrative
  privileges), \code{Polychrome} no longer imports or depends upon the
  \code{rgl} package, instead relying on \code{scatterplot3d} for its 3D
  plots. The example below shows how to plot a color set using
  \code{rgl}.
}
\value{
  Returns a list containing two components: (1) a three-column matrix
  named \code{coords} containing the luminance (L) and hue coordinates
  (U, V)   of each color provided in the input colorset; and (2) a
  character vector named \code{cset} containng a hexadecimal
  representation of the colorset.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\examples{
data(alphabet)
luv <- getLUV(alphabet)
scatterplot3d::scatterplot3d(luv$coords, color = luv$cset,
    pch = 16, cex.symbol = 3)
\dontrun{
library(rgl)
x <- luv$coords
cset <- luv$cset
open3d(windowRect=c(40, 40, 840, 840))
plot3d(x, main="Alphabet Colors")
spheres3d(x, radius=10, col=cset, shininess=100)
}
}
\keyword{color}

\name{xkcdNames}
\alias{xkcdNames}

\title{Consensus Names for Colors}
\description{
  Standardizing color names is a challenging task. Elsewhere in this
  package, we provide tools for assigning either standard names from the
  ISCC or common names from the historical "rgb.txt" files on UNIX
  computers. Here, we provide access toth e "consensus" color names
  derived froma n online survey conducted by Randall Munroe of XKCD.
}
\usage{
xkcdNames(colorset)
}
\arguments{
  \item{colorset}{A character vector containing hexadecimal
    representations of colors.}
}
\details{
  Each of the 949 XKCD consensus color names can be represented by the
  centroid of a region of CIE L*u*v* color space, all of whose points
  should be given the same name. This function takes colors represented
  in the common hexadecimal notation, maps them into L*u*v* color space,
  and assigns the name of the nearest XKCD color.
}
\value{
  A character string containing the standard color name nearest (in CIE
  L*u*v* color space) to each input color.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\references{
  \url{https://blog.xkcd.com/2010/05/03/color-survey-results/}.
}
\seealso{
  \code{\link{xkcd}}, \code{\link[grDevices]{colors}}.
}
\examples{
data(alphabet)
xkcdNames(alphabet)
colorNames(alphabet)
}
\keyword{color}

\name{sortByHue}
\alias{sortByHue}
\alias{sortByLuminance}

\title{Sorting Palettes}
\description{
  Functions to sort palettes; potentially useful for combining existing
  palettes to create new ones.
}
\usage{
sortByHue(colorset)
sortByLuminance(colorset)
}
\arguments{
  \item{colorset}{a character vector containing hexadecimal color
    values.}
}
\details{
  These functions take a palette as input, sort it either by the hue or
  by the luminance, and return the result. One possibnle aplication
  would be to combine "dark" and "light" palettes to generate larger
  version of the RColorBrewer "Paired" palette.
}
\value{
  Returns a new color set (i.e., a palette, implemented as a character
  string containing the hex values of color), after sorting.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\examples{
D <- dark.colors(24)
L <- light.colors(24)
X <- sortByHue(c(D,L))
names(X) <- colorNames(X)
X <- X[!duplicated(names(X))]
swatch(X)
Y <- sortByLuminance(X)
swatch(Y)
}
\keyword{color}

\name{xkcd}
\alias{xkcd}
\docType{data}
\title{Color Names From the Color Survey by Randall Munroe of XKCD}
\description{
  A data frame mapping hex codes for 949 colors to their consensus
  names in the XKCD color survey.
}
\usage{data(xkcd)}
\format{
  A data frame with two columns and 949 rows.
}
\details{
  This data set contains the names and hex codes for 949 color names
  identified as consistent in the Color Survey run by Randall Munroe at
  XKCD.  During the online survey, more than five million color
  swatches were named across 222,500 user sessions. In the blog post
  describing the results, Munroe describes the method he used to come up
  with a list of "consensus" color names. He also says the text file
  should contain 954 color names, but the downloaded text file only
  contained 949. Data were reformatted to be usable as a data frame in
  R, with underscores inserted in place of spaces in multi-word color
  names.
}
\source{
  Our main source was the text file linked to from
  \url{https://blog.xkcd.com/2010/05/03/color-survey-results/}.
}
\seealso{
  \code{\link{xkcdNames}}
}
\examples{
data(xkcd)
head(xkcd)
}
\keyword{datasets}
\keyword{color}

\name{isccNames}
\alias{isccNames}
\alias{colorNames}

\title{Standard Names for Colors}
\description{
  The Inter-Society Color Council, in cooperation with the United States
  National Bureau of Standards, developed a list of 267 standardized
  color names.  Many software tools (including R) also use a
  (non-standardized) list of color names derived from the original X11
  list on early UNIX systems. We provide tools to convert hexadecimal
  colors to both sets of names.
}
\usage{
isccNames(colorset)
colorNames(colorset)
}
\arguments{
  \item{colorset}{A character vector containing hexadecimal
    representations of colors.}
}
\details{
  Each of the ISCC-NBS 267 standard color names is represented by the
  centroid of a region of CIE L*u*v* color space, all of whose points
  should be given the same name.  Each of the color names listed by the
  \code{\link[grDevices]{colors}} function has an associated RGB color that can
  also be converted to L*u*v* space. These functions take colors
  represented in the common hexadecimal notation, maps them into L*u*v*
  color space, and assigns the name of the nearest ISCC centroid or
  UNIX/X11/R color. 
}
\value{
  A character string containing the standard color name nearest (in CIE
  L*u*v* color space) to each input color.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\references{
  Kelly KL. Twenty-Two Colors of Maximum Contrast. Color Eng., 1965;
  3:26--7. 

  Also see the Inter-Society Color Council web site
  (\url{https://iscc.org/}). 
}
\seealso{
  \code{\link{iscc}}, \code{\link[grDevices]{colors}}.
}
\examples{
data(alphabet)
isccNames(alphabet)
colorNames(alphabet)
}
\keyword{color}

\name{createPalette}
\alias{createPalette}

\title{Creating New Color Palettes}
\description{
  Tool to create new palettes that are well separated in CIE L*u*v*
  color space.
}
\usage{
createPalette(N, seedcolors, prefix = "NC", range = c(30, 90),
              target = c("normal", "protanope", "deuteranope", "tritanope"),
              M = 50000)
}
\arguments{
  \item{N}{An integer, the size of the palette to create.}
  \item{seedcolors}{A character vector containing the hexadecimal
    representations of one or more colors.}
  \item{prefix}{A character string to be used as a prefix to numeric
    names of the colors.}
  \item{range}{A numeric vector limitng the range of allowed luminance
    values.}
  \item{target}{A character string indicating the kind of color vision
    for which the palette is intended.}
  \item{M}{An integer; the number of random colors to generate while
    creating palettes.}
}
\details{
  Carter and Carter showed that "perceptual distinguishability" of
  colors was related to their Euclidean distance in the L*u*v* color
  space coordinates, as defined by the International Commisision on
  Illumination (CIE). The \code{createPalette} function implements a
  greedy algorithm to find colors that are well-spread-out in L*u*v*
  space.  The algorithm begins by generating a random set of 50,000
  colors; these colors are restricted to those whose luminance lies
  between 30 and 90.  Then, given one or more starting colors, the
  algorithm finds the random color that maximizes the distance to the
  closest existing color point.  This process continues until \code{N}
  colors have been selected.
}
\value{
  A character string containing the hexadecimal representations of
  \code{N} colors that are well spread out in CIE L*u*v* color space.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\references{
  Carter RC, Carter EC. High-contrast sets of colors. Applied Optics,
  1982; 21(16):2936--9.

  Coombes KR, Brock G, Abrams ZB, Abruzzo LV. Polychrome: Creating and
  Assessing Qualitative Palettes with Many Colors. Journal of
  Statistical Software. 2019; 90(1):1--23.
}
\seealso{
  \code{\link{Color Palettes}}, \code{\link{colorDeficit}}.
}
\examples{
seed <- c("#ff0000", "#00ff00", "#0000ff")
mycolors <- createPalette(15, seed, prefix="mine")
swatch(mycolors)
}
\keyword{color}

\name{palette36}
\alias{palette36}
\docType{data}
\title{A 36-Color Palette}
\description{
  A palette composed of 36 distinctive colors.  
}
\usage{data(palette36)}
\format{
  A character string of length 36.
}
\details{
  A character vector containing hexadecimal color representations of 36
  distinctive colors that are well separated in the CIE L*u*v* color
  space.  Each color is assigned a name from the ISCC-NBS standard.
}
\source{
  The color palette was generated using the \code{\link{createPalette}}
  function with three seed colors: ebony ("#5A5156"), iron ("#E4E1E3"),
  and red ("#F6222E"). 
}
\seealso{
  \code{\link{createPalette}}, \code{\link{isccNames}}.
}
\examples{
data(palette36)
palette36
}
\keyword{datasets}
\keyword{color}

\name{palettes}
\alias{palettes}
\alias{Color Palettes}
\alias{alphabet.colors}
\alias{kelly.colors}
\alias{green.armytage.colors}
\alias{palette36.colors}
\alias{glasbey.colors}
\alias{light.colors}
\alias{dark.colors}
\alias{sky.colors}

\title{Polychrome Color Palettes}
\description{
  Five color palettes each containing at least 22 different,
  distinguishable colors.
}
\usage{
kelly.colors(n = 22)
glasbey.colors(n = 32)
green.armytage.colors(n = 26)
palette36.colors(n = 36)
alphabet.colors(n = 26)
light.colors(n = 24)
dark.colors(n = 24)
sky.colors(n = 24)
}
\arguments{
  \item{n}{An integer; the number of colors desired.}
}
\details{
  Kenneth Kelly, a physicist who worked at the United States National
  Bureau of Standards and chaired the Inter-Society Color Council
  Subcommittee on Color Names, made one of the earliest attempts to find
  a set of colors that could be easily distinguished when used in
  graphs. The \code{kelly.colors} function produces a palette from the
  22 colors that he produced, using his color names.  These are ordered
  so that the optimal contrast for any palette with fewer than 22 colors
  can be selected from the top of his list.

  Glasbey and colleagues used a sequential search algorithm in CIE LAB
  color space to create a palette of 32 well-separated colors.

  Paul Green-Armytage described a study growing out of a workshop held
  by the Colour Society of Australia in 2007 to test whether an alphabet
  composed of 26 distinguishable colors would serve in place of the usual
  symbols of the English alphabet. Each color is given a name starting
  with a different letter of the alphabet, which was found to make it
  easier for people to learn the association and read sentences written
  in color. The \code{green.armytage.colors} function produces palettes
  from his final color set, arranged in "alphabetical" order rather than
  by maximum contrast.

  Carter and Carter followed Kelly's article with a study that showed
  that "perceptual distinguishability" of colors was related to their
  Euclidean distance in the L*u*v* color space coordinates, as defined
  by the International Commisision on Illumination (CIE).  They also
  found that distinguishability falls off rapidly when the distance is
  less than about 40 L*u*v* units. We implemented a palette-construction
  algorithm based on this idea.  The \code{palette36.colors} function
  returns palettes from the resulting list of 36 colors, with names
  assigned using the ISCC-NSB standard.

  The \code{alphabet.colors} function uses the first 26 colors from
  "\code{palette36}" but assigns them names beginning with different
  letters of the English alphabet and reorders them accordingly.

  The \code{light.colors} and \code{dark.colors} functions use one of
  the two 24-color palettes (\code{Light24} or \code{Dark24}) customized
  to limit the luminance range.

  The \code{sky.colors} function uses the 24-color palette constructed
  by Coombes et al. to match as closely as possibkle te palette used by
  the standard software useed by cytogeneticists to display the results
  of spectral karyotyping.
}
\value{
  Each function returns a character vector of hexadecimal color values
  (such as "#EA9399"). Each color is assigned a name (such as
  "Strong_Pink").  The default value is the maximum number of colors
  available from the individual palette.
}
\author{Kevin R. Coombes <krc@silicovore.com>}
\references{
  Kelly KL. Twenty-Two Colors of Maximum Contrast. Color Eng., 1965;
  3:26--7.

  Green-Armytage, P. A Colour Alphabet and the Limits of Colour
  Coding. Colour: Design and Creativity, 2010; 10:1--23.

  Carter RC, Carter EC. High-contrast sets of colors. Applied Optics,
  1982; 21(16):2936--9.

  Coombes KR, Brock G, Abrams ZB, Abruzzo LV. Polychrome: Creating and
  Assessing Qualitative Palettes with Many Colors. Journal of
  Statistical Software. 2019; 90(1):1--23.

}
\seealso{
  \code{\link{createPalette}}
}
\examples{
palette36.colors(5)
kelly.colors(5)
alphabet.colors(7)
glasbey.colors(9)
green.armytage.colors(3)
light.colors(6)
dark.colors(11)
sky.colors(4)
}
\keyword{color}

\name{sky-colors}
\alias{sky-colors}
\alias{Spectral Karyotyping}
\docType{data}
\title{The 32-color Glasbye palette}
\description{
  A palette composed of 32 distinct colors.
}
\usage{data("sky-colors")}
\format{
  A character string of length 24.
}
\details{
  A character vector containing hexadecimal color representations of 24
  distinctive colors that are well separated in the CIE L*u*v* color
  space.
}
\source{
  Spectral karyotyping is a standard cytogenetic technology to map
  chromosomes using multiple different colored fluorescent probes. The
  probes for each individual human chromosome use different combinations
  of one or more probes. The standard software to display the result
  uses a false-color mapping to a 24-color paletee. Coombes et al. (see
  references) showed that the common colorm palette includes several
  colors that are hard to distinguish, then genrated this palette as a
  suggested replacement. The copde to generate the palette (and to match
  it as closely as possible with the older standard) can be found inth e
  appendix to that paper
}
\references{
  Coombes KR, Brock G, Abrams ZB, Abruzzo LV. Polychrome: Creating and
  Assessing Qualitative Palettes with Many Colors. Journal of
  Statistical Software. 2019; 90(1):1--23.
}
\examples{
data("sky-colors")
sky.colors
}
\keyword{datasets}
\keyword{color}
